####### data reading and preprocessing #######

f440 <- readShapeSpatial("../2007/440_2007.shp") 
#f440 <- readShapeSpatial("../final-datensaetze/f440.shp")
f611 <- readShapeSpatial("../2007/611_2007.shp") 
#f611 <- readShapeSpatial("../final-datensaetze/f611.shp") 
# newly edited original data, 2011-01-18
f610 <- readShapeSpatial("../2008/S610/610-2008-punkte-auswertung.shp")
#f610 <- readShapeSpatial("../final-datensaetze/f610.shp")
f631 <- readShapeSpatial("../2008/S631/631-ertrag-strat-em38rueck1und3plus plan.shp")
#f631 <- readShapeSpatial("../final-datensaetze/f631.shp")
#f631 <- readShapeSpatial("../2008/S631/GK4/631-2008-punkte-auswertung.shp")
f550 <- readShapeSpatial("../S550/gesamt_2GK_New_POINTS.shp")
#f550 <- readShapeSpatial("../final-datensaetze/f550.shp")

##### things to set per data frame

# this sets the reference system for the coordinates in the
# shape files (important!)
proj4string(f440) <- CRS("+init=epsg:31468")
proj4string(f550) <- CRS("+init=epsg:31468")
proj4string(f611) <- CRS("+init=epsg:31468")
proj4string(f610) <- CRS("+init=epsg:31468")
proj4string(f631) <- CRS("+init=epsg:4326") 

### if necessary, swap some coordinates in f440 and f611 
# (is also recently swapped in the standards, funny circumstance)
hochwert.f440 <- as.numeric(as.character(f440@coords[,1]))
rechtswert.f440 <- as.numeric(as.character(f440@coords[,2]))
f440@coords[,1] <- rechtswert.f440
f440@coords[,2] <- hochwert.f440 
hochwert.f611 <- as.numeric(as.character(f611@coords[,1]))
rechtswert.f611 <- as.numeric(as.character(f611@coords[,2]))
f611@coords[,1] <- rechtswert.f611
f611@coords[,2] <- hochwert.f611 


# if necessary:
# this transforms the coordinates to another CRS 
#f440 <- spTransform(f440,CRS("+init=epsg:31468"))
#f550 <- spTransform(f550,CRS("+init=epsg:2398"))
#f610 <- spTransform(f610,CRS("+init=epsg:2398"))
#f611 <- spTransform(f611,CRS("+init=epsg:2398"))
f631 <- spTransform(f631,CRS("+init=epsg:31468"))


# if necessary: delete long/lat from dataframe
# (both values are stored in the coords anyway)
f440$LONGITUDE <- NULL
f440$LATITUDE <- NULL
f611$LONGITUDE <- NULL
f611$LATITUDE <- NULL
f631$RECHTSWERT <- NULL
f631$HOCHWERT<- NULL


# set the variables which are of no further interest right now
# (for example for yield prediction)

excludelist.f440 <- list("MWShapeID","THEME","LONGITUDE","LATITUDE","STRATEGIE","HOEHE", "x","y","z","ID")
#excludelist.f440 <- list("MWShapeID","SORTE","THEME","LONGITUDE","LATITUDE","STRATEGIE","HOEHE", "x","y","z","ID")
excludelist.f611 <- list("MWShapeID","SORTE","THEME","LONGITUDE","LATITUDE","STRATEGIE","HOEHE", "x","y","z","ID")
excludelist.f610 <- list("HOEHE", "x","y","z","ID","GESCHWIND","MDNR","VG","VGNUM","RUECK1_NAS","N2_PLAN","RUECK3_NAS","VG.1","VG_NUM","N1_PLAN","N3_PLAN")
excludelist.f631 <- list("HOEHE", "x","y","z","ID","GESCHWIND","MDNR","VG","VGNUM","RUECK1_NAS","N2_PLAN","RUECK3_NAS","VG.1","VG_NUM","N1_PLAN","N3_PLAN")
excludelist.f550 <- list("LAYER", "X","Y","STRATEGIE")
#"z","ID","GESCHWIND","MDNR","VG","VGNUM","RUECK1_NAS","N2_PLAN","RUECK3_NAS","VG.1","VG_NUM","N1_PLAN","N3_PLAN")

preprocsppdataframe <- function(sppdf,excludelist){

        # add long/lat/height to data frame as numeric variable instead of factor variable
        #coords <- cbind(as.numeric(as.character(sppdf$LATITUDE)),
        #                as.numeric(as.character(sppdf$LONGITUDE)),
        #                as.numeric(as.character(sppdf$HOEHE)))
        #sppdf@data$x <- coords[,1]
        #sppdf@data$y <- coords[,2]
        #sppdf@data$z <- coords[,3]
        
        # calculate n nearest neighbors for points
        # knn <- knearneigh(coords[,1:2], k=4)
        # listofneighbors <-knn2nb(knn)
        # sppdf@data$nblist <-nb2listw(listofneighbors)

        # remove certain variables from the predictors
        predictors <- names(sppdf@data)
        predictors_to_remove <- match(excludelist, predictors)
        #predictors <- names(f440@data)
        #predictors_to_remove_f440 <- match(excludelist, predictors)
        
        # prepare for converting the factor variables to numeric variables
        # predictors: just the variables we'd like to use for yield prediction
        for(j in predictors_to_remove){predictors[j] <- list(NULL)}
        predictors <- predictors[!sapply(predictors,is.null)]
        
        # convert to numeric variables
        for(i in predictors){
                if (!(i %in% excludelist)){
                sppdf[[i]] <- as.numeric(as.character(sppdf[[i]]))
                }
        }

        #remove some outliers (REIP49=0)
        if ("REIP49" %in% colnames(sppdf@data))
        { sppdf <- sppdf[sppdf$REIP49 > 0, ] }

        return(list(sppdf=sppdf,predictors=predictors))
}

# this function corrects the yield measurements using the 
# moisture measurements; standard moisture is 0.14 (14%)
# added for F610/F631/F550 on 2011-01-18 russ@iws
moisturecorrection <- function(sppdf,yieldvar,moistvar){
        newyield <- sppdf[yieldvar]@data * (1- (sppdf[moistvar]@data -14)/(100-14))
        sppdf[[yieldvar]] <- unlist(newyield)
        return(sppdf)
}

f611 <- preprocsppdataframe(f611,excludelist.f611)
f611pred <- f611$predictors
f611 <- f611$sppdf
f440 <- preprocsppdataframe(f440,excludelist.f440)
f440pred <- f440$predictors
f440 <- f440$sppdf
f610 <- preprocsppdataframe(f610,excludelist.f610)
f610pred <- f610$predictors
f610 <- f610$sppdf
f631 <- preprocsppdataframe(f631,excludelist.f631)
f631pred <- f631$predictors
f631 <- f631$sppdf

# f440 rename some predictors (=introduce new var and remove old one)
f440$ELEVATION <- as.numeric(levels(f440$HOEHE)[f440$HOEHE])
f440$HOEHE <- NULL

# f550 ##################################################
# one NA line in the soil sampling part, remove that record
f550 <- f550[!is.na(f550$MG_MG_100G), ]
f550 <- moisturecorrection(f550,"ERTRAG03","MOIST03")
f550 <- moisturecorrection(f550,"ERTRAG04","MOIST04")
f550 <- moisturecorrection(f550,"ERTRAG07","MOIST07")
f550 <- preprocsppdataframe(f550,excludelist.f550)
f550pred <- f550$predictors
f550 <- f550$sppdf

f550$K <- f550$K_MG_100G
f550$K_MG_100G <- NULL
f550$Mg <- f550$MG_MG_100G
f550$MG_MG_100G <- NULL
f550$P <- f550$P_MG_100G
f550$P_MG_100G <- NULL
f550$pH<- f550$PH_WERT
f550$PH_WERT <- NULL
f550$ELEVATION <- f550$HOEHE
f550$ELEVATION <- f550$HOEHE
f550$HOEHE <- NULL
f550$YIELD03 <- f550$ERTRAG03
f550$ERTRAG03 <- NULL
f550$YIELD04 <- f550$ERTRAG04
f550$ERTRAG04 <- NULL
f550$YIELD07 <- f550$ERTRAG07
f550$ERTRAG07 <- NULL

# f611 ##################################################
f611$ELEVATION <- as.numeric(levels(f611$HOEHE)[f611$HOEHE])
f611$HOEHE <- NULL

# f610 ############################################
# moisture correction for f610
f610 <- moisturecorrection(f610,"ERTRAG","KORNFEU")

f610$ELEVATION <- f610$HOEHE
f610$HOEHE <- NULL
f610$YIELD07 <- f610$ERTRAG
f610$ERTRAG <- NULL
f610$N1 <- f610$N1_RUECK
f610$N1_RUECK <- NULL
f610$N2 <- f610$N2_RUECK
f610$N2_RUECK <- NULL
f610$N3 <- f610$N3_RUECK
f610$N3_RUECK <- NULL

# f631 ############################################
# moisture correction for f631
f631 <- moisturecorrection(f631,"ERTRAG","KORNFEU")
f631$ELEVATION <- f631$HOEHE
f631$HOEHE <- NULL
f631$YIELD07 <- f631$ERTRAG
f631$ERTRAG <- NULL
f631$N1 <- f631$RUECK1_NAS
f631$RUECK1_NAS <- NULL
f631$N2 <- f631$N2_PLAN
f631$N2_PLAN <- NULL
f631$N3 <- f631$RUECK3_NAS
f631$RUECK3_NAS <- NULL


# create prediction formulae for data sets
#f440formula <- as.formula( paste("YIELD07", "~", "N1 + N2 + N3 + EC25 + REIP32 + REIP49 + STRATEGY", sep=""))
#f550formula <- as.formula( paste( "YIELD04", "~", "N1 + N2 + N3 + EC25 + REIP32 + REIP49 + ELEVATION", sep=""))
#f550formula2007 <- as.formula( paste( "YIELD07", "~", "K + Mg + P + pH + ELEVATION", sep=""))
#f611formula <- as.formula( paste("YIELD07", "~", "N1 + N2 + N3 + EC25 + REIP32 + REIP49 + ELEVATION", sep=""))
#f610formula <- as.formula( paste("YIELD07", "~", "N1 + N2 + N3 + EC25 + ELEVATION", sep=""))
#f631formula <- as.formula( paste("YIELD07", "~", "N1 + N2 + N3 + EC25 + ELEVATION", sep=""))


# create additional files for external purposes (saga) with different coordinates
# 
#f440t <- spTransform(f440,CRS("+proj=longlat +datum=WGS84"))
writeSpatialShape(f440,"~/tmp/r-export-f440")
#f550t <- spTransform(f550,CRS("+proj=longlat +datum=WGS84"))
writeSpatialShape(f550,"~/tmp/r-export-f550")
#f610t <- spTransform(f610,CRS("+proj=longlat +datum=WGS84"))
writeSpatialShape(f610,"~/tmp/r-export-f610")
#f611t <- spTransform(f611,CRS("+proj=longlat +datum=WGS84"))
writeSpatialShape(f611,"~/tmp/r-export-f611")
#f631t <- spTransform(f631,CRS("+proj=longlat +datum=WGS84"))
writeSpatialShape(f631,"~/tmp/r-export-f631")
#f631 <- readShapeSpatial("../2008/S631/631-ertrag-strat-em38rueck1und3plus plan.shp")
