#### displaying certain parts of the data frames

makePredictorPlots <- function(sppdf, framename, predictors, groups, cex){

        # all eps graphics in one file
        # postscript(file = paste(framename, "-predictorplots.eps", sep = ""), onefile = TRUE )

        # following: multiple eps files as output
        
        for(i in predictors){
            postscript(file = paste("results/predictorplots/", framename, "-predictorplot-", i, ".eps", sep = ""), onefile = TRUE,
                 width=8, height=8, paper="special", family="ComputerModern", horizontal=FALSE)
                # compute cuts (quantile() doesn't work since quantiles are non-unique)
                cuts <- cut2(sppdf@data[[i]], g=groups, onlycuts=TRUE) 

                #print(spplot(sppdf,zcol = i, do.log=TRUE, key.space=list(x=-0.6,y=0.1,corner=c(0,1)),  cuts = cuts,
                #print(spplot(sppdf,zcol = i, do.log=TRUE, key.space=list(x=0, y=0.01, corner=c(0,0)),  cuts = cuts,
                tp <- trellis.par.get()
                tp$axis.text$cex <- 1.3
                tp$par.xlab.text$cex <- 1.3
                tp$par.ylab.text$cex <- 1.3
                tp$par.main.text$cex <- 1.3
                trellis.par.set(tp)

                print(spplot(sppdf, zcol = i, 
                        #do.log=TRUE, 
                        key.space=list(x=0,y=0.01, corner=c(0,0)),
												#cuts=cuts,
                        xlab = "Easting", ylab = "Northing", scales=list(draw=T), cex=cex,
                        main = paste(framename, i, sep = " ")))
            dev.off()
            print(paste("making predictor plot for ",framename, ":",i, sep=""))
        }
}

f550plots <- c("ELEVATION","EC25","REIP32", "REIP49", "N1", "N2", "N3", "K", "Mg", "P", "pH",
               "YIELD03", "YIELD04", "YIELD07")
makePredictorPlots(f440, "f440", names(f440)[6:24], groups=8, cex=0.7)

#makePredictorPlots(f440, "f440", "SORTE", groups=8, cex=0.7)
#makePredictorPlots(f550, "f550", c("STRATEGIE"), groups=1, cex=1.5)
#makePredictorPlots(f610, "f610", all.vars(f610formula), groups=8, cex=0.7)
#makePredictorPlots(f611, "f611", all.vars(f611formula), groups=8, cex=0.5)
#makePredictorPlots(f631, "f631", all.vars(f631formula), groups=8, cex=0.5)
#makePredictorPlots(f550, "f550", all.vars(f550formula2007), groups=10, cex=1.5)

## stuff for calculating Moran's I
# mostly from http://listproc.ucdavis.edu/archives/geospatial/log0806/0001.html
# transform coordinates from factor to numeric

makeMoranScatterplots <- function(sppdf, framename, predictors){
        x <- sppdf@coords[,1]
        y <- sppdf@coords[,2]

        knn <- knearneigh(cbind(x,y), k=4)
        neighborslist<-knn2nb(knn)
        neighborslist <-nb2listw(neighborslist)

                # one postscript file for all plots
        #        postscript(file = paste("results/", framename, "-moranscatterplot.eps", sep = ""), onefile = TRUE )

        for(i in predictors){
                #if (!(i %in% excludelist)){
                      # one postscript file per plot
															#paper="special", 
                      postscript(file = paste("results/", framename, "-moranscatter-", i, ".eps", sep = ""), 
															onefile = FALSE, family="ComputerModern")
        							par(mar=c(6,5,5,3) + 0.1)
                      print(moran.plot(x = sppdf@data[[i]], listw = neighborslist, labels = FALSE,
														cex.axis=2,
														cex.lab=2,
														cex.main=2,
														xlab = i, 
														ylab = paste("spatially lagged values of ",i,sep = ""), 
                            main = paste("Moran Scatterplot for ", framename, ", ", i, sep = "")
														)
											)
                      #main = paste("Moran Scatterplot for ", framename, ", ", i, sep = ""), cex)
                      #title(xlab = i, ylab = paste("spatially lagged values of ",i,sep = ""), cex.lab=2,
                #}
                dev.off()
        }
}

#makeMoranScatterplots(f440, "F440", "EC25")
#makeMoranScatterplots(f611, "F611", f611pred, excludelist, onefile = FALSE)

## calculate variogram
# function: variogram in package gstat
# from http://www.ats.ucla.edu/stat/R/faq/variogram.htm
# and http://www.gstat.org/s.html

# create semivariogram plots from georeferenced data contained
# in a SpatialPointsDataFrame, for all predictor variables in 
# the data frame
makeVariograms <- function(sppdataframe,framename,predictors,excludelist){

        # all eps graphics in one file
        # postscript(file = paste(framename, "-variograms.eps", sep = ""), onefile = TRUE )
        # multiple eps files as output
        #postscript(file = paste(framename, "-variograms-%03d.eps", sep = ""), horizontal = TRUE, onefile = FALSE)

        #if (onefile)
                # one postscript file for all plots
        #        postscript(file = paste("results/", framename, "-variogram.eps", sep = ""), onefile = TRUE )
        #else
                # one postscript file per plot

        x <- as.numeric(sppdataframe@coords[,1])
        y <- as.numeric(sppdataframe@coords[,2])

        for(i in predictors){
                print(paste("making variogram for: ", framename, ", ", i, sep="" ))
                postscript(file = paste("results/", framename, "-variogram-", i, ".eps", sep = ""),
													 width=6, height=6, paper="special", family="ComputerModern")
                if (!(i %in% excludelist)){
                        #variogramt <- variogram(sppdataframe@data[[i]] ~1, locations = ~ x + y, data = sppdataframe@data)
                        variogramt <- variogram(sppdataframe@data[[i]] ~x+y, data = sppdataframe)
                        fonts <- trellis.par.get()
                        fonts$axis.text$cex <- 1.8
                        fonts$par.xlab.text$cex <- 2.3
                        fonts$par.ylab.text$cex <- 2.3
                        fonts$par.main.text$cex <- 2.5
                        trellis.par.set(fonts)
                        # specification of plot details is in xyplot (variogram is plotted via xyplot)
                        pp <- plot(variogramt, main = paste("Semivariogram of ", i, " on ", framename, sep=""),
                        						xlab = paste("distance in m", sep=""), ylab = "semivariance", type="l", scales=list(cex=5))
                        update(pp,scales=list(cex=5))
                        print(pp)
                }
       				dev.off()
        }
}

#makeVariograms(f440, framename="f440", predictors=f440pred, excludelist=excludelist.f440)
#makeVariograms(f611, framename="f611", f611pred, excludelist.f611)
#makeVariograms(f631, "F631", f631pred, excludelist.f631, onefile = FALSE)
#makeVariograms(f610, "F610", f610pred, excludelist.f610, onefile = FALSE)

#%mcm <- function(sppdf,framename,predictors){
#%
#%				x <- sppdf@coords[,1]
#%				y <- sppdf@coords[,2]
#%				l <- pretty(range(sppdf@data[i]), 10)
#%        postscript(file = paste("results/", framename, "-contourmap-", i, ".eps", sep = ""), horizontal = TRUE,
#%									     onefile = FALSE, 
#%											 #width=8, height=8, 
#%											 paper="special",
#%											 family="ComputerModern")
#%					# par(bg="lightcyan")
#%					# pin <- par("pin")
#%					# xdelta <- diff(range(x))
#%					# ydelta <- diff(range(y))
#%					# xscale <- pin[1]/xdelta
#%					# yscale <- pin[2]/ydelta
#%					# scale <- if(xscale < yscale) xscale else yscale
#%					# xadd <- 0.5*(pin[1]/scale-xdelta)
#%					# yadd <- 0.5*(pin[2]/scale-ydelta)
#%					# plot(numeric(0), numeric(0),
#%							      xlim=range(x)+c(-1,1)*xadd, ylim=range(y)+c(-1,1)*yadd,
#%										     type="n", ann=FALSE)
#%					# usr <- par("usr")
#%					#rect(usr[1], usr[3], usr[2], usr[4], col="green3")
#%					contour(x, y, sppdf@data[i], levels=l, col="yellow", lty="solid", add=TRUE)
#%					box()
#%					title("A Topographic Map of Maunga Whau",font=4)
#%					title(xlab="Meters North", ylab="Meters West", font=3)
#%					mtext("10 Meter Contour Spacing", side=3, line=0.35, outer=FALSE,
#%								      at=mean(par("usr")[1:2]), cex=0.7, font=3)
#%				dev.off()
#%
#%}
#%
#%mcm(f440,"f440",predictors=c("N3"))

makeContourMap <- function(sppdf,framename,predictors,excludelist){

        # most from http://casoilresource.lawr.ucdavis.edu/drupal/book/export/html/442
#        postscript(file = paste("results/", framename, "-contourmap.eps", sep = ""))
        x.range <- as.integer(range(sppdf@coords[,1]))
        y.range <- as.integer(range(sppdf@coords[,2]))
        grd <- expand.grid(x=seq(from=x.range[1], to=x.range[2], by=10), y=seq(from=y.range[1], to=y.range[2], by=10) )
        coordinates(grd) <- ~x+y
        gridded(grd) <- TRUE

        # I'm using x and y for the coordinates, 
        # and I'm too lazy to change the script below,
        # so I just add x and y here for later use.
        # 2010-08-12, norge@georgruss.de
        sppdf$x <- sppdf@coords[,1]
        sppdf$y <- sppdf@coords[,2]

        
        # remove values from grid which are outside of the field (na.pts)
        # determine hull
        hullpts <- chull(sppdf@coords[,1], sppdf@coords[,2]) # get indices of hull points
        hullpts <- c(hullpts,hullpts[1]) # connect last to first point (make closed polygon)
        na.pts <- cbind(sppdf@coords[hullpts,1],sppdf@coords[hullpts,2]) # get actual point coordinates
        na.sppol <- SpatialPolygons(list(Polygons(list(Polygon(na.pts)),ID="1"))) # get polygon (correct class for overlay)
        ov <- overlay(grd,na.sppol) # calculate overlay points 

        for(i in predictors){
                if (!(i %in% excludelist)){
            						print(paste("making contour plot for ",framename, ":",i, sep=""))
                        postscript(file = paste("results/", framename, "-contourmap-", i, ".eps", sep = ""),
															horizontal = TRUE, onefile = FALSE, 
															#paper="special", 
															family="ComputerModern")
                        #currenti <- sppdf@data[i]
                        g <- gstat(id="pred", formula=as.formula(paste(i,"~1")), locations = ~x+y, data=sppdf@data)
												print(paste("getting experimental variogram"))
                        v <- variogram(g, alpha = c(0,45,90,135))
												print(paste("fitting linear variogram"))
                        v.fit <- fit.variogram(v, model=vgm(model='Lin', anis=c(0, 0.5)))
                        print(plot(v, model=v.fit, as.table=TRUE))
                        g <- gstat(g, id="pred", model=v.fit)

												print(paste("kriging"))
                        # kriging (takes time)
                        p <- predict(g, model=v.fit, newdata=grd)

                        # set points outside the field polygon to NA
                        p@coords[is.na(ov)] <- NA 
                        p@data$pred.pred[is.na(ov)] <- NA
                        p@data$pred.var[is.na(ov)] <- NA
        
												print(paste("plotting figure"))
        								par(mar=c(6,5,5,3) + 0.1)
                        contour(p, axes = TRUE, nlevels=5, labcex=1.5, cex.axis=2)
                        title(main=paste(framename, i, sep=" "), cex.lab=2, xlab="Easting", ylab="Northing")
                        dev.off()
                }
        }
        return(p)
}

#pn3 <- makeContourMap(f440, "f440", predictors=c("N3"), excludelist.f440)
#pec25 <- makeContourMap(f440, "f440", predictors=c("EC25"), excludelist.f440)

# function for partitioning the field (spatialpointsdataframe) into 
# k regions automatically using k-means clustering
# returns: list (=set) of lists (=clusters) of data point indices that make up each cluster
makeTessellation <- function(sppdf,framename,k,returncenters){
        
        points <- sppdf@coords[,1:2]
        clustering <- kmeans(points,k)

        # rest of the function: for generating the field's plot
        # generate color sequence (one color for each cluster)
        at = seq(1,k,1)
        # for color clusters
        # col = rainbow(length(at))
        # for gray-level clusters
        # a <- (c(1:k)*0.8)/k # this excludes colors which are too "white"
        col = gray((c(1:k)*0.8)/k) 

        postscript(file = paste("results/", framename, "-tessellationplot-",sprintf("%03d",k),".eps", sep = ""), 
                onefile = FALSE, width=8, height=8, paper="special", family="ComputerModern", horizontal=FALSE)

        spl <- list() 
        sppd <- list() # list of data frames that make up the tessellation

        # iterate over all clusters,
        # this loop is mainly for preparing the color figures 
        # with the field's partitioning 
        for(j in 1:k) {
              # get indices of points in current cluster
              cc <- which(clustering$cluster==j)
              # get according coordinates
              spp <- SpatialPoints(sppdf@coords[cc,1:2])
              sppd[[j]] <- cc

              # for the spatial layout function in the plot below
              # for each cluster in the partition store the respective
              # points/color in a list
              spl[[j]] <- list("sp.points", spp, col = col[j], pch=19, cex=1.0)
        }

        # set graphical parameters (font size)
         tp <- trellis.par.get()
         tp$axis.text$cex <- 1.3
         tp$par.xlab.text$cex <- 1.3
         tp$par.ylab.text$cex <- 1.3
         tp$par.main.text$cex <- 1.3
         trellis.par.set(tp)


        # finally, print plot: all points first in "transparent color"
        # and then the clustering in "sp.layout" with colors on the top layer
              print(
                spplot(sppdf, zcol = names(sppdf)[1], 
                        #do.log=TRUE,
                        auto.key=FALSE,
                        #key.space=list(x=0, y=0.01, corner=c(0,0)),  
                        xlab = "Easting", ylab = "Northing", 
                        main = paste(framename, ", ", sprintf("%03d",k), " clusters", sep = ""),
                        col.regions = "transparent",
                        sp.layout = spl)
               )

                # compute cuts (quantile() doesn't work since quantiles are non-unique)
               # cuts <- cut2(sppdf@data[[i]], g=groups, onlycuts=TRUE) 

                #print(spplot(sppdf,zcol = i, do.log=TRUE, key.space=list(x=-0.6,y=0.1,corner=c(0,1)),  cuts = cuts,
                #print(spplot(sppdf,zcol = i, do.log=TRUE, key.space=list(x=0, y=0.01, corner=c(0,0)),  cuts = cuts,
                #print(spplot(sppdf, zcol = i, do.log=TRUE, key.space="right", cuts=cuts,
                #xlab = "Easting", ylab = "Northing", scales=list(draw=T), cex=cex,
                #main = paste(framename, i, sep = " ")))


       # the key/legend still exists, but can be removed in latex via 
       # trim and clip in \includegraphics options

       dev.off()
       if(returncenters == FALSE){
        return(sppd)}
        else{
        sppd$clustercenters <- clustering$center
        return(sppd)
        #return(clustering$center)
        }
}

#a <- makeTessellation(f440,"f440",20, returncenters=FALSE)
#b <- makeTessellation(f611,"F611",20, returncenters=FALSE)
