#source("01-packs-options.R")
#source("02-preproc.R")
#source("05-rsaga-dem-f440.R")
#source("10-functions-shapefile.R")

# stuff for average linkage hierarchical agglomerative constrained clustering
# all functions for average linkage clustering are
# prefixed with "avr"

# for debugging purposes
options(digits=2)

avrPrepareClusters <- function(sppdf, framename, predictors){

        clf <-  list()
        clf$data <- sppdf
        clf$data.norm <- clf$data

        for (i in predictors){
             clf$data.norm[[i]] <- scale(clf$data[[i]])
        }

        length <- dim(sppdf)[1]

				# list(s) of clusters (will be updated throughout the algorithm)
				clusters <- list()
				for (i in 1:length){
								clusters[i] <- i
								names(clusters)[i] <- paste(i)
				}
				clf$clusters <- clusters

				# project data frame to the actual variables in use
        projectedframe <- as.matrix(clf$data.norm[,predictors]@data)

				print("generating distance matrix")
        dists <- dist(projectedframe, method="euclidean")
				clf$distmat <- as.matrix(dists)
        dimnames(clf$distmat) <- list(c(1:dim(sppdf)[1]), c(1:dim(sppdf)[1]))

				# generate full distance matrix (not just lower half)
				clf$distmat <- clf$distmat + t(clf$distmat) 
				diag(clf$distmat) <- NA

				# generate adjacency (neighbours) matrix (from library tripack)
				print("generating adjacency (neighbours) matrix")
			  neighbors <- tri2nb(sppdf@coords)
				neighmat <- nb2mat(neighbors, style="B")

        dimnames(neighmat) <- list(c(1:dim(sppdf)[1]), c(1:dim(sppdf)[1]))

				clf$neighmat <- replace(neighmat, neighmat == 0, NA)
        return(clf)
}

avrDetermineClosestClusters  <- function(clf, constraint){
				# cc: closestClusters

				if(constraint==TRUE){
								constrainedmat  <- clf$distmat*clf$neighmat
								minvalue <- min(constrainedmat, na.rm=TRUE) 
								index <- which(constrainedmat == minvalue, arr.ind=TRUE)
								cc <- list(cl1 = paste(dimnames(clf$distmat)[[1]][index[1,1]]), 
											 cl2=paste(dimnames(clf$distmat)[[1]][index[1,2]]),
											 cl1ind = index[1,1],
											 cl2ind = index[1,2])
								rm(constrainedmat)
				}
				else{
					minvalue <- min(clf$distmat, na.rm=TRUE)
					index  <- which(clf$distmat == minvalue, arr.ind=TRUE)
					cc <- list(cl1 = paste(dimnames(clf$distmat)[[1]][index[1,1]]),
											 cl2=paste(dimnames(clf$distmat)[[1]][index[1,2]]),
											 cl1ind = index[1,1],
											 cl2ind = index[1,2])
				}

				# count commas in cluster name
			  # (deparsing, since cluster size is needed later)
				# cluster size = number of commas in cluster name + 1
				cl1size <- length(gregexpr(",",cc$cl1)[[1]][1])
				cl2size <- length(gregexpr(",",cc$cl2)[[1]][1])

				if(cl1size == -1){
								cc$cl1size <- 1
				}else{
								cc$cl1size <- cl1size+1
				}
				if(cl2size == -1){
								cc$cl2size <- 1
				}else{
								cc$cl2size <- cl2size+1
				}

				# determine mean feature space distance of adjacent clusters
				adjmean  <- (sum(rowMeans(clf$distmat*clf$neighmat,na.rm=TRUE)))/(2*(dim(clf$distmat)[1]))
				cc$adjmean <- adjmean
				cc$adjmin <- minvalue
				print(paste("adjmean: ", adjmean, sep=""))

				# determine mean feature space distance of non-adjacent clusters
				nonneighmat <- replace(clf$neighmat, is.na(clf$neighmat), 2)
				nonneighmat <- replace(nonneighmat, nonneighmat==1, NA)
				nonneighmat <- replace(nonneighmat, nonneighmat==2, 1)
				diag(nonneighmat) <- NA
				nonadjmean  <- (sum(rowMeans(clf$distmat*nonneighmat,na.rm=TRUE)))/(2*(dim(clf$distmat)[1]))
				cc$nonadjmean <- nonadjmean
				#print(nonadjmean)
				rm(nonneighmat)

				return(cc)
}

avrUpdateNeighMat <- function(clf, cc){
				# cc: closestClusters

				print("updating neigh mat")

				neighscl1 <- clf$neighmat[cc$cl1,]
				neighscl2 <- clf$neighmat[cc$cl2,]

				matSize <- dim(clf$neighmat)[1]
				
				newneighs <- pmax(neighscl1, neighscl2, na.rm=TRUE)
				newneighs <- newneighs[-c(cc$cl1ind,cc$cl2ind)]

#				print("old neighmat:")
#				print(clf$neighmat)
				# delete old neighbor rows/columns
				clf$neighmat <- clf$neighmat[-c(cc$cl1ind,cc$cl2ind),-c(cc$cl1ind,cc$cl2ind)]
				#clf$neighmat <- clf$neighmat[,-c(cc$cl1ind,cc$cl2ind)]

				# glue new neighbors vector to neighbors matrix
				clf$neighmat <- rbind(clf$neighmat,newneighs)

				# add an NA to the newneighs vector (because the matrix
				# contains NAs on the diagonal and because it's one 
				# line longer due to the step above)
				newneighs <- c(newneighs, NA)
				clf$neighmat <- cbind(clf$neighmat,newneighs)

				# set name of new cluster
				dimnames(clf$neighmat)[[2]][matSize-1] <- paste("(",cc$cl1,",",cc$cl2,")",sep="")
				dimnames(clf$neighmat)[[1]] <- dimnames(clf$neighmat)[[2]]
				
				return(clf)
}

avrUpdateDistMat <- function(clf,cc){
				print("updating avr distance matrix between two clusters")
				# distance matrix update
				distscl1 <- clf$distmat[cc$cl1,]
				distscl2 <- clf$distmat[cc$cl2,]

				# average distance calculation
				newdists <- (distscl1*cc$cl1size + distscl2*cc$cl2size)/(cc$cl1size+cc$cl2size)

				matSize <- dim(clf$distmat)[1]

				# remove values from the original diagonal 
				# which were copied to the vectors
				newdists <- newdists[-c(cc$cl1ind,cc$cl2ind)]

				# delete old neighbor rows/columns
				clf$distmat <- clf$distmat[-c(cc$cl1ind,cc$cl2ind),-c(cc$cl1ind,cc$cl2ind)]
				#clf$distmat <- clf$distmat[,-c(cc$cl1ind,cc$cl2ind)]

				# glue new neighbors vector to neighbors matrix
				clf$distmat <- rbind(clf$distmat,newdists)

				# add an NA to the newneighs vector (because the matrix
				# contains NAs on the diagonal and because it's one 
				# line longer due to the step above)
				newdists <- c(newdists, NA)
				clf$distmat <- cbind(clf$distmat,newdists)

				dimnames(clf$distmat)[[2]][matSize-1] <- paste("(",cc$cl1,",",cc$cl2,")",sep="")
				dimnames(clf$distmat)[[1]] <- dimnames(clf$distmat)[[2]]

				return(clf)
}

avrUpdateClusterLists <- function(clf,cc){
				pointscl1 <- clf$clusters[[cc$cl1]]
				pointscl2 <- clf$clusters[[cc$cl2]]
				clf$clusters[[cc$cl1]] <- NULL
				clf$clusters[[cc$cl2]] <- NULL
				clf$clusters[[paste("(",cc$cl1,",",cc$cl2,")",sep="")]] <- c(pointscl1, pointscl2)
				#dimnames(clf$distmat)[[1]][matSize-1] <- paste("(",cc$cl1,",",cc$cl2,")",sep="")
				return(clf)
}


avrHACC <- function(sppdf, framename, predictors, filetype, contigfact, color, cex){

	clf  <- avrPrepareClusters(sppdf,framename, predictors)
	#clf  <- avrPrepareClusters(sppdf[1:300,],framename, predictors)
	clusters <- dim(sppdf)[1]
  seqavrdist <- array(data =NA, dim=c(3,clusters))
	returnlist <- list()
	constraint <- TRUE
	constraintSwitchedOffAt <- NA

				for (j in c(1:(clusters-1))){
				#for (j in c(1:299)){
							print(j)
							
							cc  <- avrDetermineClosestClusters(clf, constraint=constraint)
							seqavrdist[1,j] <- cc$adjmean
							seqavrdist[2,j] <- cc$nonadjmean
							seqavrdist[3,j] <- cc$adjmin

							print(cc$adjmean)
							print(cc$nonadjmean)
							print(cc$adjmean/cc$nonadjmean)
							if (!is.na(cc$adjmean) && !is.na(cc$nonadjmean)){
								if(((cc$adjmean/cc$nonadjmean) > contigfact) && constraint==TRUE){
												constraint=FALSE
												print(paste(" ##################### constraint switched at: ", j, sep=""))
												constraintSwitchedOffAt <- j
								}
							}

							clf <- avrUpdateNeighMat(clf, cc)
							clf <- avrUpdateDistMat(clf, cc)
							clf <- avrUpdateClusterLists(clf, cc)
					    avrPlotClusters(clf, framename, numclust=clusters, predictors=predictors,
										 filetype=filetype, contigfact=contigfact, number=j, color=color, cex=cex)
				}

        returnlist$clf <- clf
        returnlist$seqavrdist <- seqavrdist
        returnlist$constraintSwitchedOffAt <- constraintSwitchedOffAt

        # construct filename containing the parameters (in subdir)
        file <- paste("results/clusterings/", framename, "-cf-", contigfact, "-clustering-", clusters, "-",
											paste(predictors, collapse="-"), sep="")
        save(returnlist, file=paste(file, ".Rdata", sep=""))

        return(returnlist)
}

avrPlotClusters <- function(clf, framename, numclust, predictors,filetype, contigfact, number, color, cex){
				 
				 k <- 0
				 # get total number of colors
         for(j in names(clf$clusters)) {
								 cc <- clf$clusters[[j]]
								 if(length(cc) > 1){
												 k <- k +1
							}
				 }

         noofclusters <- length(clf$clusters)
				#print(paste("length(clf$clusters): ", k, sep=""))

         at <- seq(1,k,1)

         if (color == TRUE){
                # for color clusters
                col <- rainbow(length(at))
         }
         else {
                # for gray-level clusters
                col = gray((c(1:k)*0.8)/k)
                #col = gray(1:(k-1) / k)
         }

         spl <- list()

         colorcounter <- 0


         for(j in names(clf$clusters)) {


              # get indices of points in current cluster
              cc <- clf$clusters[[j]]
              # get according coordinates
				 			if(length(cc) > 1){
              	colorcounter <- colorcounter + 1
								#print(paste("colorcounter: ", colorcounter, sep=""))
              	spp <- SpatialPoints(clf$data@coords[cc,1:2])

              	# for the spatial layout function in the plot below
              	# for each cluster in the partition store the respective
              	# points/color in a list
              	spl[[j]] <- list("sp.points", spp, col = col[colorcounter], pch=19, cex=cex)
							} 
							}

        if(filetype == "postscript"){
                postscript(file = paste("results/clusterplots/", framename, "/", framename, 
                "-hsc-cf-",  contigfact, 
                "-", paste(predictors,collapse="-"), 
                "-cl", numclust,
                "-", sprintf("%04d",number),".eps", sep = ""))
                }

        if(filetype == "png"){
                png(file=paste("results/clusterplots/",framename, "/", framename,
                "-hsc-cf-", contigfact, 
                "-", paste(predictors,collapse="-"),
                "-cl", numclust,
                "-", sprintf("%04d",number),".png", sep=""))
                }

        # Make sure there's an "EC25" variable, otherwise the spplot won't display.
        # If there's no EC25 variable, choose a different one and adapt
        # the function below accordingly
        # hint above superseded, just taking the first predictor in "predictors"

        print(
                spplot(clf$data, zcol = predictors[1],
                #do.log=TRUE,
                #key.space=list(x=0, y=0.01, corner=c(0,0)),
                xlab = "Easting", ylab = "Northing",
                main = paste(framename, ", ", noofclusters, " zones, ", paste(predictors), collapse = "-"),
                col.regions = "transparent", 
                sp.layout = spl)
        )
        dev.off()
}


####################
# this function takes the clustering results and creates some figures and
# evaluation from it

plotClusteringResults <- function(clr,framename, contigfact, predictors){

				file=paste("results/clusterings/",framename, "-cf-", contigfact, "-",
															paste(predictors, collapse="-"),"-clustering-ratio-plots-%02d.eps",sep="")
        postscript(file=file, onefile=FALSE,
                        width=6,height=3,paper="special", family="ComputerModern", horizontal=FALSE)
        #################

        par(las=1)

        # unsmoothed
        plot(clr$seqavrdist[1,],
                main=paste(framename, paste(predictors, collapse="-"), "average distances of adjacent clusters",sep=", "),
                xlab="cluster steps",
                ylab="distance",
                #ylim=c(-3,0),
        #        par(las=0),
                type="l")
                if(!(is.na(clr$constraintSwitchedOffAt))){abline(v=clr$constraintSwitchedOffAt)}
        plot(clr$seqavrdist[2,],
                main=paste(framename, paste(predictors, collapse="-"), "average distances of non-adjacent clusters",sep=", "),
                xlab="cluster steps",
                ylab="distance",
                #ylim=c(-3,0),
        #        par(las=0),
                type="l")
                if(!(is.na(clr$constraintSwitchedOffAt))){abline(v=clr$constraintSwitchedOffAt)}
        plot(clr$seqavrdist[1,]/clr$seqavrdist[2,],
                main=paste(framename, paste(predictors, collapse="-"), "contiguity ratio",sep=", "),
                xlab="cluster steps",
                ylab="ratio",
                #ylim=c(0,4),
        #        par(las=0),
                type="l")
                if(!(is.na(clr$constraintSwitchedOffAt))){abline(v=clr$constraintSwitchedOffAt)}
        # non-smoothed and smoothed values in one diagram

        #nSMD <- clr$nonSpatMedianDist
        # clip spatMedianDist at the end, since nSMD is necessarily shorter
        #sMD <- clr$spatMedianDist[1:length(nSMD)]

        #par(las=1, xpd=TRUE,mar=par()$mar+c(0,0,0,8))

        dev.off()

}
